/* Last edited on 2012-03-05 10:12:25 by stolfilocal */
/* See {interp_spline.h}. */

#define _GNU_SOURCE
#include <stdio.h>
#include <math.h>
#include <assert.h>

#include <bool.h>
#include <indexing.h>
#include <affirm.h>

#include <interp_spline.h>
#include <interp_spline_I.h>
#include <interp_spline_B.h>
#include <interp_spline_O.h>

int interp_spline_compute_num_samples(int order, interp_spline_kind_t kind)
   { switch(kind)
      { 
        case interp_spline_kind_B:
          return interp_spline_B_compute_num_samples(order);
        case interp_spline_kind_I:
          return interp_spline_I_compute_num_samples(order);
        case interp_spline_kind_O:
          return interp_spline_O_compute_num_samples(order);
        default:
          demand(FALSE, "invalid {kind}");
          return 0;
      }
  }
  
void interp_spline_get_indices(double t, int N, ix_reduction_t red, int m, int i[])
  {
    bool_t debug = FALSE;

    /* Get the raw index of the first pixel: */
    int iz = (int)floor(t - 0.5*(m - 2));
    
    /* Compute the reduced indices {i[0..m-1]}: */
    int k;
    for (k = 0; k < m; k++)
      { i[k] = ix_reduce(iz, N, red);
        iz++;
      }

    if (debug)
      { fprintf(stderr, "t = %7.4f", t);
        for (k = 0; k < m; k++) { fprintf(stderr, "  i[%d] = %d\n", k, i[k]); }
        fprintf(stderr, "\n");
      }
  }
  
void interp_spline_get_weights(double t, int order, interp_spline_kind_t kind, int m, double w[])
  {
    switch(kind)
      { 
        case interp_spline_kind_B:
          interp_spline_B_get_weights(t, order, m, w);
          break;
        case interp_spline_kind_I:
          interp_spline_I_get_weights(t, order, m, w);
          break;
        case interp_spline_kind_O:
          interp_spline_O_get_weights(t, order, m, w);
          break;
        default:
          demand(FALSE, "invalid {kind}");
      }
  }
