#ifndef interp_spline_H
#define interp_spline_H

/* Spline smoothing/interpolation of equally-spaced numerical series. */
/* Last edited on 2012-03-04 19:51:01 by stolfilocal */ 

#include <bool.h>
#include <indexing.h>

typedef enum 
  { interp_spline_kind_B,    /* B-splines (convex, broad window, not interpolating). */
    interp_spline_kind_I,    /* I-splines (interpolating, min window, not convex). */
    interp_spline_kind_O,    /* O-splines (interpolating, min degree, not convex). */
    interp_spline_kind_NUM   /* Number of spline kinds. */
  } interp_spline_kind_t;

int interp_spline_compute_num_samples(int order, interp_spline_kind_t kind);
  /* Computes the number of samples {m} needed along each axis for
    spline interpolation specified by {order} and {kind}. */

void interp_spline_get_indices(double t, int N, ix_reduction_t red, int m, int i[]);
  /* Computes the indices {i[0..m-1]} of the {m} pixels needed to perform
    interpolation in a sequence of samples at the fractional coordinate
    {t}. Indices {i[k]} that fall outside the range {0..N-1} are reduced
    according to {ix_reduce(i, N, red)}. Some reduced indices {i[k]} may
    be {-1} to denote `no such pixel'. */

void interp_spline_get_weights(double t, int order, interp_spline_kind_t kind, int m, double w[]);
  /* Computes the weights {w[0..m-1]} of the {m} pixels needed to
    perform spline interpolation defined by {order} and {kind}
    in a sequence of samples at the fractional
    coordinate {t}. The size {m} of {w} must be correct for
    the given {order} and {kind}. */

#endif
