/* Last edited on 2012-03-04 19:56:58 by stolfilocal */
/* See {interp_spline_B.h}. */

#define _GNU_SOURCE
#include <stdio.h>
#include <assert.h>
#include <math.h>

#include <affirm.h>
#include <bool.h>

#include <interp_spline_B.h>
 
int interp_spline_B_compute_num_samples(int order)
  {
    return order+2;
  }
 
void interp_spline_B_get_weights(double t, int order, int m, double w[])
  {
    bool_t debug = FALSE;
    assert(m == order+2);
    w[0] = 1.0;
    if (order >= 0)
      { /* Adjust {t} depending on parity of {order}: */
        t = t - 0.5*(order + 1);

        /* Get the fraction {fz}: */
        int iz = (int)floor(t);
        double fz = t - iz;
        assert(fz >= 0.0);
        assert(fz < 1.0);

        /* Compute the fraction's complement {gz}: */
        double gz = 1.0 - fz;

        /* Compute the interpolation weights {w[0..m-1]}: */
        int j, k;
        for (k = 0; k <= order; k++)
          { w[k+1] = w[k]*fz;
            for (j = k; j > 0; j--) { w[j] = w[j]*gz + w[j-1]*fz; }
            w[0] = w[0]*gz;
          }
      }
    if (debug)
      { fprintf(stderr, "t = %7.4f", t);
        int k;
        for (k = 0; k < m; k++) { fprintf(stderr, "  w[%d] = %10.7f\n", k, w[k]); }
        fprintf(stderr, "\n");
      }
  }
