 /* Last edited on 2012-03-04 22:04:49 by stolfilocal */
/* See {interp_spline_I.h}. */

#define _GNU_SOURCE
#include <stdio.h>
#include <math.h>
#include <assert.h>

#include <affirm.h>
#include <bool.h>

#include <interp_spline_I.h>

int interp_spline_I_compute_num_samples(int order)
  {
    switch(order)
      { 
        case -1:
          return 1;
        case 0:
          return 2;
        case 1:
          return 4;
        default:
          fprintf(stderr, "%s(%d) not implemented yet\n", __FUNCTION__, order);
          return 0;
          // if (order == 0)
          //   { return 2; }
          // else if (order == 1)
          //   { return 4; }
          // else
          //   { demand(FALSE, "invalid interpolation order"); }
      }
  }
 
void interp_spline_I_get_weights(double t, int order, int m, double w[])
  {
    bool_t debug = FALSE;
    if (order == -1)
      { 
        assert(m == 1);
        w[0] = 1.0;
      }
    else
      { 
        /* Adjust {t} to be relative to start of first pixel: */
        t = t - 0.5*(m - 1);

        /* Get the raw index of the first pixel: */
        int iz = (int)floor(t);

        /* Get the fraction {fz} and its complement {gz}: */
        double fz = t - iz;
        assert(fz >= 0.0);
        assert(fz < 1.0);
        double gz = 1.0 - fz;
        
        if (debug) { fprintf(stderr, "t = %9.6f  fz = %9.6f  gz = %9.6f", t, fz, gz); }

        switch(order)
          { 
          case 0:
            assert(m == 2);
            w[0] = gz;
            w[1] = fz;
            break;
          case 1:
            assert(m == 4);
            if (fz < 0.5) {
              w[0] = ((+0.75)*fz - 0.5)*fz;
              w[1] = (-1.75)*fz*fz + 1;
              w[2] = ((+1.25)*fz + 0.5)*fz;
              w[3] = (-0.25)*fz*fz;
            } else {
              w[0] = (-0.25)*gz*gz;
              w[1] = ((+1.25)*gz + 0.5)*gz;
              w[2] = (-1.75)*gz*gz + 1;
              w[3] = ((+0.75)*gz - 0.5)*gz;
            }
            break;
          case 2:
            break;
          default: 
            demand(FALSE, "not implemented yet");  
          }
      }
    
    if (debug)
      { int k;
        for (k = 0; k < m; k++) { fprintf(stderr, "  w[%d] = %10.7f\n", k, w[k]); }
        fprintf(stderr, "\n");
      }
  }

// # I-spline with 12 pieces of degree 3, hopefully of order 2
//   D0I2x0(z) = (((+0.003472)*z+0.000000)*z-0.000000)*z+0.000000
//   D0I2x1(z) = (((-0.024306)*z+0.010417)*z+0.010417)*z+0.003472
//   D0I2x2(z) = (((+0.031250)*z-0.062500)*z-0.041667)*z-0.000000
//   D0I2x3(z) = (((+0.114583)*z+0.031250)*z-0.072917)*z-0.072917
//   D0I2x4(z) = (((-0.138889)*z+0.375000)*z+0.333333)*z-0.000000
//   D0I2x5(z) = (((-0.194444)*z-0.041667)*z+0.666667)*z+0.569444
//   D0I2x6(z) = (((+0.194444)*z-0.625000)*z-0.000000)*z+1.000000
//   D0I2x7(z) = (((+0.138889)*z-0.041667)*z-0.666667)*z+0.569444
//   D0I2x8(z) = (((-0.114583)*z+0.375000)*z-0.333333)*z-0.000000
//   D0I2x9(z) = (((-0.031250)*z+0.031250)*z+0.072917)*z-0.072917
//   D0I2x10(z) = (((+0.024306)*z-0.062500)*z+0.041667)*z-0.000000
//   D0I2x11(z) = (((-0.003472)*z+0.010417)*z-0.010417)*z+0.003472
//   D0I2R(x) = ((x < -6.0)||(x > +6.0) ? 0 : (x < +0.0 ? (x < -3.0 ? (x < -4.0 ? (x < -5.0 ? D0I2x0(x+6.0) : D0I2x1(x+5.0)) : D0I2x2(x+4.0)) : (x < -1.0 ? (x < -2.0 ? D0I2x3(x+3.0) : D0I2x4(x+2.0)) : D0I2x5(x+1.0))) : (x < +3.0 ? (x < +2.0 ? (x < +1.0 ? D0I2x6(x-0.0) : D0I2x7(x-1.0)) : D0I2x8(x-2.0)) : (x < +5.0 ? (x < +4.0 ? D0I2x9(x-3.0) : D0I2x10(x-4.0)) : D0I2x11(x-5.0)))))
//   D0I2(x) = D0I2R(x/0.5)
//   I2(x) = D0I2(x)
