#define PROG_NAME "test_interp_spline"
#define PROG_DESC "test of {float_image_interpolate_C0.h} and {float_image_interpolate_C1.h}"
#define PROG_VERS "1.0"

/* Last edited on 2012-03-04 21:59:44 by stolfilocal */ 
/* Created on 2012-03-04 by J. Stolfi, UNICAMP */

#define test_interp_spline_COPYRIGHT \
  "Copyright  2012  by the State University of Campinas (UNICAMP)"

#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <assert.h>

#include <interp_spline.h>

#include <bool.h>
#include <jsfile.h>
#include <affirm.h>
#include <indexing.h>

int main(int argn, char **argv);

void do_plot_tests(char *prefix);

void do_plot_test(char *prefix, int order, interp_spline_kind_t kind);
  /* Writes a file with a plot of a sample vector interpolated with the specified {order}
    and {kind}, and its derivatives. */
    
void generate_test_samples(int *nP, double **sP);
  /* Generates a vector {s[0..n-1]} of samples to be interprolated,
    returns {n} in {*nP} and {s} in {*sP}. */

int main (int argc, char **argv)
  {
    demand(argc == 2, "wrong num of parameters");
    char *prefix = argv[1];
    
    do_plot_tests(prefix);

    return 0;
  }
 
#define order_MAX 3
#define deg_MAX 3

void do_plot_tests(char *prefix)
  {
    int order, kind;
    for (order = -1; order <= order_MAX; order++)
      { for (kind = 0; kind < interp_spline_kind_NUM; kind++)
          { do_plot_test(prefix, order, kind); }
      }
  }

void do_plot_test(char *prefix, int order, interp_spline_kind_t kind)
  {
    int m = interp_spline_compute_num_samples(order, kind);
    if (m == 0) { return; }
    
    ix_reduction_t red = ix_reduction_SINGLE;
    
    int NS;
    double *s;
    generate_test_samples(&NS, &s);
    
    double w[m];    /* Interpolation weights. */
    int i[m];       /* Sample indices. */
    
    auto double interp(double z);
    
    double interp(double z)
      { interp_spline_get_indices(z, NS, red, m, i);
        interp_spline_get_weights(z, order, kind, m, w);
        double sum_ws, sum_w = 0;
        int j;
        for (j = 0; j < m; j++) 
          { if (i[j] >= 0) { sum_ws += w[j]*s[i[j]]; sum_w += w[j]; } }
        return sum_ws/sum_w;
      }
    
    /* Plot the interpolation: */
    int NU = 20;    /* Num subssamples per sample. */
    int NP = NU*NS; /* Num subsamples. */

    char *fname = NULL;
    asprintf(&fname, "out/%s-r%c-o%c-k%c.txt", prefix, "SERMP"[red], (order < 0 ? 'm' : '0'+order), "BIO"[kind]);
    FILE *plot = open_write(fname, TRUE);
    int k;
    for (k = 0; k < NP; k++)
      { double z = ((double)k)/((double)NU);
        double Fz = interp(z);
        double Sz = (k % NU == 0 ? s[k/NU] : NAN);
        fprintf(plot, "%10.6f %+12.6f %+12.6f\n", z, Fz, Sz);        
      }
    fclose(plot);
    /* Cleanup: */
    free(fname);
    free(s);
  }  

void generate_test_samples(int *nP, double **sP)
  {
    /* Generate the data to be interpolated: */
    int HT = 10;  /* Half-width of each test segment. */
    int HW =  6;  /* Max half-width of pulse. */
    int DX =  3;  /* Space between test segments. */
    int NS = DX+HW+1+HW+(deg_MAX+1)*(DX+HW+2*HT+1+HW)+DX;
    double *s = notnull(malloc(NS*sizeof(double)), "no mem");
    
    auto void test_segm(int *kP, int g, int H);
    
    void test_segm(int *kP, int g, int H)
      { int j;
        for (j = -H; j <= +H; j++)
          { double x = ((double)j)/((double)H);
            s[(*kP)] = pow(x, g);
            (*kP)++;
          }
      }
    
    int i;
    for (i = 0; i < NS; i++) { s[i]= 0; }
    
    int k = 0;
    k += DX;
    k += HW;
    s[k] = 1.0; k++;
    k += HW;
    int g;
    for (g = 0; g <= deg_MAX; g++)
      { k += DX;
        k += HW;
        test_segm(&k,g,HT);
        k += HW;
      }
    k += DX;
    assert(k == NS);

    (*nP) = NS;
    (*sP) = s;
  }
